@@ -1,8 +1,10 @@
+/* $Cambridge: exim/src/src/dbfunctions.h,v 1.6 2009/11/16 19:50:36 nm4 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 
@@ -19,10 +21,15 @@ int      dbfn_write(open_db *, uschar *, void *, int);
 
 #define  dbfn_read(a, b) dbfn_read_with_length(a, b, NULL)
 
-/* Berkeley DB uses a callback function to pass back error details. */
+/* Berkeley DB uses a callback function to pass back error details. Its API
+changed at release 4.3. */
 
 #if defined(USE_DB) && defined(DB_VERSION_STRING)
+#if DB_VERSION_MAJOR > 4 || (DB_VERSION_MAJOR == 4 && DB_VERSION_MINOR >= 3)
+void     dbfn_bdb_error_callback(const DB_ENV *, const char *, const char *);
+#else
 void     dbfn_bdb_error_callback(const char *, char *);
 #endif
+#endif
 
 /* End of dbfunctions.h */
