@@ -1,8 +1,10 @@
+/* $Cambridge: exim/src/src/rewrite.c,v 1.5 2009/11/16 19:50:37 nm4 Exp $ */
+
 /*************************************************
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2003 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Functions concerned with rewriting headers */
@@ -12,7 +14,7 @@
 
 /* Names for testing rewriting */
 
-static char *rrname[] = {
+static const char *rrname[] = {
   "  sender",
   "    from",
   "      to",
@@ -27,20 +29,20 @@ static char *rrname[] = {
 
 typedef struct where_list_block {
   int bit;
-  uschar *string;
+  const uschar *string;
 } where_list_block;
 
 static where_list_block where_list[] = {
-  { rewrite_sender,  US"sender:" },
-  { rewrite_from,    US"from:" },
-  { rewrite_to,      US"to:" },
-  { rewrite_cc,      US"cc:" },
-  { rewrite_bcc,     US"bcc:" },
-  { rewrite_replyto, US"reply-to:" },
-  { rewrite_envfrom, US"env-from" },
-  { rewrite_envto,   US"env-to" },
-  { rewrite_smtp,    US"smtp recipient" },
-  { rewrite_smtp|rewrite_smtp_sender, US"smtp sender" }
+  { rewrite_sender,  CUS"sender:" },
+  { rewrite_from,    CUS"from:" },
+  { rewrite_to,      CUS"to:" },
+  { rewrite_cc,      CUS"cc:" },
+  { rewrite_bcc,     CUS"bcc:" },
+  { rewrite_replyto, CUS"reply-to:" },
+  { rewrite_envfrom, CUS"env-from" },
+  { rewrite_envto,   CUS"env-to" },
+  { rewrite_smtp,    CUS"smtp recipient" },
+  { rewrite_smtp|rewrite_smtp_sender, CUS"smtp sender" }
 };
 
 static int where_list_size = sizeof(where_list)/sizeof(where_list_block);
@@ -162,8 +164,8 @@ for (rule = rewrite_rules;
     /* Use the general function for matching an address against a list (here
     just one item, so use the "impossible value" separator UCHAR_MAX+1). */
 
-    if (match_address_list(subject, FALSE, &(rule->key), NULL, 0,
-        UCHAR_MAX + 1) != OK)
+    if (match_address_list(subject, FALSE, TRUE, &(rule->key), NULL, 0,
+        UCHAR_MAX + 1, NULL) != OK)
       continue;
 
     /* The source address matches, and numerical variables have been
@@ -246,10 +248,11 @@ for (rule = rewrite_rules;
 
   /* We have a validly rewritten address */
 
-  if ((log_selector & L_address_rewrite) != 0)
+  if ((log_write_selector & L_address_rewrite) != 0 ||
+      (debug_selector & D_rewrite) != 0)
     {
     int i;
-    uschar *where = US"?";
+    const uschar *where = CUS"?";
 
     for (i = 0; i < where_list_size; i++)
       {
@@ -431,7 +434,7 @@ qualified addresses, but there are options to permit unqualified ones from
 certain hosts. For those hosts only, addresses in headers can also be
 qualified. For other hosts, unqualified addresses in headers do not get touched
 in any way. For locally sourced messages, unqualified addresses always get
-qualified.
+qualified, except when -bnq is used to explicitly suppress this.
 
 Arguments:
   h              pointer to header line block
@@ -453,6 +456,7 @@ rewrite_one_header(header_line *h, int flag, uschar *routed_old,
 {
 int lastnewline = 0;
 header_line *newh = NULL;
+void *function_reset_point = store_get(0);
 uschar *s = Ustrchr(h->text, ':') + 1;
 while (isspace(*s)) s++;
 
@@ -464,14 +468,16 @@ parse_allow_group = TRUE;     /* Allow group syntax */
 /* Loop for multiple addresses in the header. We have to go through them all
 in case any need qualifying, even if there's no rewriting. Pathological headers
 may have thousands of addresses in them, so cause the store to be reset for
-any that don't actually get rewritten. */
+any that don't actually get rewritten. We also play silly games for those that
+_are_ rewritten so as to avoid runaway store usage for these kinds of header.
+We want to avoid keeping store for any intermediate versions. */
 
 while (*s != 0)
   {
   uschar *sprev;
   uschar *ss = parse_find_address_end(s, FALSE);
   uschar *recipient, *new, *errmess;
-  void *reset_point = store_get(0);
+  void *loop_reset_point = store_get(0);
   BOOL changed = FALSE;
   int terminator = *ss;
   int start, end, domain;
@@ -490,7 +496,11 @@ while (*s != 0)
   /* There isn't much we can do for syntactic disasters at this stage.
   Pro tem (possibly for ever) ignore them. */
 
-  if (recipient == NULL) continue;
+  if (recipient == NULL)
+    {
+    store_reset(loop_reset_point);
+    continue;
+    }
 
   /* If routed_old is not NULL, this is a rewrite caused by a router,
   consisting of changing routed_old into routed_new, and applying to all
@@ -534,7 +544,10 @@ while (*s != 0)
 
     if (changed && ((is_recipient && !allow_unqualified_recipient) ||
                     (!is_recipient && !allow_unqualified_sender)))
+      {
+      store_reset(loop_reset_point);
       continue;
+      }
     }
 
   /* If there are rewrite rules for this type of header, apply
@@ -559,23 +572,34 @@ while (*s != 0)
       }
     }
 
+  /* If nothing has changed, lose all dynamic store obtained in this loop, and
+  move on to the next address. We can't reset to the function start store
+  point, because we may have a rewritten line from a previous time round the
+  loop. */
+
+  if (!changed) store_reset(loop_reset_point);
+
   /* If the address has changed, create a new header containing the
   rewritten address. We do not need to set the chain pointers at this
-  stage. */
+  stage. We want to avoid using more and more memory if the header is very long
+  and contains lots and lots of rewritten addresses. Therefore, we build the
+  new text string in malloc store, then at the end we reset dynamic store
+  before copying the new header to a new block (and then freeing the malloc
+  block). The header must end up in dynamic store so that it's freed at the end
+  of receiving a message. */
 
-  if (changed)
+  else
     {
     int remlen;
     int newlen = Ustrlen(new);
     int oldlen = end - start;
 
     header_line *prev = (newh == NULL)? h : newh;
-    header_line *newnewh = store_get(sizeof(header_line));
-    uschar *newt = store_get(prev->slen - oldlen + newlen + 4);
+    uschar *newt = store_malloc(prev->slen - oldlen + newlen + 4);
+    uschar *newtstart = newt;
 
-    newnewh->text = newt;
-    newnewh->type = prev->type;
-    newnewh->slen = prev->slen - oldlen + newlen;
+    int type = prev->type;
+    int slen = prev->slen - oldlen + newlen;
 
     /* Build the new header text by copying the old and putting in the
     replacement. This process may make the header substantially longer
@@ -604,15 +628,15 @@ while (*s != 0)
     another one! The pointer s has been skipped over white space, so just
     look back to see if the last non-space-or-tab was a newline. */
 
-    if (newlen > oldlen && newt - newnewh->text - lastnewline > 40)
+    if (newlen > oldlen && newt - newtstart - lastnewline > 40)
       {
       uschar *p = s - 1;
       while (p >= prev->text && (*p == ' ' || *p == '\t')) p--;
       if (*p != '\n')
         {
-        lastnewline = newt - newnewh->text;
+        lastnewline = newt - newtstart;
         Ustrcat(newt, "\n\t");
-        newnewh->slen += 2;
+        slen += 2;
         }
       }
 
@@ -621,19 +645,30 @@ while (*s != 0)
     Ustrcat(newt, s);
 
     DEBUG(D_rewrite) debug_printf("newlen=%d newtype=%c newtext:\n%s",
-      newnewh->slen, newnewh->type, newnewh->text);
+      slen, type, newtstart);
 
-    s = newnewh->text + (s - prev->text) - oldlen + newlen;
+    /* Compute the length of the rest of the header line before we possibly
+    flatten a previously rewritten copy. */
 
+    remlen = (s - prev->text) - oldlen + newlen;
+
+    /* We have the new text in a malloc block. That enables us to release all
+    the memory that has been used, back to the point at which the function was
+    entered. Then set up a new header in dynamic store. This will override a
+    rewritten copy from a previous time round this loop. */
+
+    store_reset(function_reset_point);
+    newh = store_get(sizeof(header_line));
+    newh->type = type;
+    newh->slen = slen;
+    newh->text = string_copyn(newtstart, slen);
+    store_free(newtstart);
+
+    /* Set up for scanning the rest of the header */
+
+    s = newh->text + remlen;
     DEBUG(D_rewrite) debug_printf("remainder: %s", (*s == 0)? US"\n" : s);
-
-    newh = newnewh;
     }
-
-  /* If the address was not changed, reset so as to reuse the store it
-  was extracted into. */
-
-  else store_reset(reset_point);
   }
 
 parse_allow_group = FALSE;  /* Reset group flags */
@@ -671,7 +706,7 @@ The old header line is flagged as "old". Old headers are saved on the spool for
 debugging but are never sent to any recipients.
 
 Arguments:
-  h               header line to rewrite
+  h              header line to rewrite
   routed_old     if not NULL, this is a rewrite caused by a router, changing
                    this domain into routed_new
   routed_new     new routed domain if routed_old is not NULL
